/*
 * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package jdk.nashorn.internal.codegen;

import jdk.nashorn.internal.ir.FunctionNode;
import jdk.nashorn.internal.parser.Token;
import jdk.nashorn.internal.parser.TokenType;
import jdk.nashorn.internal.runtime.PropertyMap;
import jdk.nashorn.internal.runtime.ScriptFunction;
import jdk.nashorn.internal.runtime.Source;

/**
 * A container for data needed to instantiate a {@link ScriptFunction} at runtime.
 */
public class ScriptFunctionData {

    private final String      name;
    private final Source      source;
    private final PropertyMap allocatorMap;
    private final long        token;
    private final int         arity;
    private final boolean     needsCallee;
    private final boolean     isVarArg;
    private final boolean     isStrict;

    /**
     * Constructor
     * @param fn the function node
     * @param allocatorMap the allocator property map
     */
    public ScriptFunctionData(final FunctionNode fn, final PropertyMap allocatorMap) {

        final long      firstToken = fn.getFirstToken();
        final long      lastToken  = fn.getLastToken();
        final int       position   = Token.descPosition(firstToken);
        final int       length     = Token.descPosition(lastToken) - position + Token.descLength(lastToken);

        this.name             = fn.isAnonymous() ? "" : fn.getIdent().getName();
        this.source           = fn.getSource();
        this.allocatorMap     = allocatorMap;
        this.token            = Token.toDesc(TokenType.FUNCTION, position, length);
        this.arity            = fn.getParameters().size();
        this.needsCallee      = fn.needsCallee();
        this.isVarArg         = fn.isVarArg();
        this.isStrict         = fn.isStrictMode();
    }

    /**
     * Get the arity of the function.
     * @return the arity
     */
    public int getArity() {
        return arity;
    }

    /**
     * Get the function name.
     * @return function name
     */
    public String getName() {
        return name;
    }

    /**
     * Get the source of the function.
     * @return the source
     */
    public Source getSource() {
        return source;
    }

    /**
     * Get the allocator property map.
     * @return the allocator map
     */
    public PropertyMap getAllocatorMap() {
        return allocatorMap;
    }

    /**
     * Get the function's parse token.
     * @return the token
     */
    public long getToken() {
        return token;
    }

    /**
     * Returns true if the function needs a callee argument.
     * @return the needsCallee flag
     */
    public boolean needsCallee() {
        return needsCallee;
    }

    /**
     * Returns true if this is a strict-mode function.
     * @return the strict flag
     */
    public boolean isStrict() {
        return isStrict;
    }

    /**
     * Returns true if this is a var-arg function.
     * @return the var-arg flag
     */
    public boolean isVarArg() {
        return isVarArg;
    }

}
